from pathlib import Path
import os
import site
import sys


def activate(path: Path):
    """Activate virtualenv for current interpreter:
    This can be used when you must use an existing Python interpreter, not the virtualenv bin/python.
    """

    abs_file = str(path.absolute())

    bin_dir = os.path.dirname(abs_file)
    base = bin_dir[: -len("bin") - 1]  # strip away the bin part from the __file__, plus the path separator

    # prepend bin to PATH (this file is inside the bin directory)
    os.environ["PATH"] = os.pathsep.join([bin_dir] + os.environ.get("PATH", "").split(os.pathsep))
    os.environ["VIRTUAL_ENV"] = base  # virtual env is right above bin directory

    # add the virtual environments libraries to the host python import mechanism
    prev_length = len(sys.path)
    for lib in "../lib/python3.6/site-packages".split(os.pathsep):
        path = os.path.realpath(os.path.join(bin_dir, lib))
        site.addsitedir(path.decode("utf-8") if "" else path)
    sys.path[:] = sys.path[prev_length:] + sys.path[0:prev_length]

    sys.real_prefix = sys.prefix
    sys.prefix = base



import site
from pathlib import Path
import sys
import os

WINDOWS = sys.platform.startswith("win") or (sys.platform == "cli" and os.name == "nt")
_MACOS = sys.platform == "darwin"
_HOME_SYSENV = "PKM_HOME"
_DATA_DIR_NAME = 'pkm'


def data_dir() -> Path:
    if os.getenv(_HOME_SYSENV):
        return Path(os.getenv(_HOME_SYSENV)).expanduser()

    if WINDOWS:
        const = "CSIDL_APPDATA"
        path = os.path.normpath(_get_win_folder(const))
        path = os.path.join(path, _DATA_DIR_NAME)
    elif _MACOS:
        path = os.path.expanduser(f"~/Library/Application Support/{_DATA_DIR_NAME}")
    else:
        path = os.getenv("XDG_DATA_HOME", os.path.expanduser("~/.local/share"))
        path = os.path.join(path, _DATA_DIR_NAME)

    return Path(path)


def bin_dir() -> Path:
    if os.getenv(_HOME_SYSENV):
        return Path(os.getenv(_HOME_SYSENV), "bin").expanduser()

    user_base = site.getuserbase()

    if WINDOWS:
        bin_dir = os.path.join(user_base, "Scripts")
    else:
        bin_dir = os.path.join(user_base, "bin")

    return Path(bin_dir)


def _get_win_folder_from_registry(csidl_name):
    # noinspection PyCompatibility
    import winreg as _winreg

    shell_folder_name = {
        "CSIDL_APPDATA": "AppData",
        "CSIDL_COMMON_APPDATA": "Common AppData",
        "CSIDL_LOCAL_APPDATA": "Local AppData",
    }[csidl_name]

    key = _winreg.OpenKey(
        _winreg.HKEY_CURRENT_USER,
        r"Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders",
    )
    dir, type = _winreg.QueryValueEx(key, shell_folder_name)

    return dir


def _get_win_folder_with_ctypes(csidl_name):
    import ctypes

    csidl_const = {
        "CSIDL_APPDATA": 26,
        "CSIDL_COMMON_APPDATA": 35,
        "CSIDL_LOCAL_APPDATA": 28,
    }[csidl_name]

    buf = ctypes.create_unicode_buffer(1024)
    ctypes.windll.shell32.SHGetFolderPathW(None, csidl_const, None, 0, buf)

    # Downgrade to short path name if have highbit chars. See
    # <http://bugs.activestate.com/show_bug.cgi?id=85099>.
    has_high_char = False
    for c in buf:
        if ord(c) > 255:
            has_high_char = True
            break
    if has_high_char:
        buf2 = ctypes.create_unicode_buffer(1024)
        if ctypes.windll.kernel32.GetShortPathNameW(buf.value, buf2, 1024):
            buf = buf2

    return buf.value


if WINDOWS:
    try:
        from ctypes import windll  # noqa

        _get_win_folder = _get_win_folder_with_ctypes
    except ImportError:
        _get_win_folder = _get_win_folder_from_registry



import os
import sys
from pathlib import Path
from typing import Callable, NoReturn

from pkm_installer import directories, venvs
import importlib_metadata as mtd
from virtualenv import cli_run as venv_create
import subprocess

from pkm_installer.directories import WINDOWS

_VERSION_FILE = "etc/pkm/version"
_PYTHON_EXECUTABLE = "bin/python"
_PKM_EXECUTABLE = "bin/rp" if not WINDOWS else "bin/rp.exe"
EXECUTOR_T = Callable[[], NoReturn]


def _required_version():
    return mtd.version("pkm-installer")


def _installed_version(venv: Path) -> str:
    vfile = venv / _VERSION_FILE
    if not vfile.exists():
        return ''
    return vfile.read_text()


def install() -> EXECUTOR_T:
    data_dir = directories.data_dir()
    venv_dir = data_dir / 'venvs/pkm'
    required_version = _required_version()

    if venv_dir.exists():
        installed_version = _installed_version(venv_dir)
        print(f"installed version: {installed_version}")

        if installed_version != required_version:
            _install(venv_dir, required_version)

    else:
        venv_dir.parent.mkdir(parents=True, exist_ok=True)
        venv_create([str(venv_dir)])
        _install(venv_dir, required_version)

    def execute() -> NoReturn:
        os.execve(str(venv_dir / _PKM_EXECUTABLE), sys.argv, os.environ)

    return execute


def _install(venv: Path, version: str):
    python_executable = str(venv / _PYTHON_EXECUTABLE)
    print("Needs to setup my own virtual environment after upgrade/first-install... ")
    subprocess.check_call([python_executable, "-m", "pip", "install", 'pip', '-U'])
    subprocess.check_call([python_executable, "-m", "pip", "install", f'pkm-main=={version}'])
    version_file = venv / _VERSION_FILE
    version_file.parent.mkdir(parents=True, exist_ok=True)
    version_file.write_text(version)
    print("Setup completed.")
